@@ -17,7 +17,6 @@ from fontTools.otlLib import builder as otl
 from contextlib import contextmanager
 from operator import setitem
 import logging
-import warnings
 
 class MtiLibError(Exception): pass
 class ReferenceNotFoundError(MtiLibError): pass
@@ -1074,7 +1073,7 @@ class Tokenizer(object):
 		if len(line) == 1 and not line[0]:
 			del line[0]
 		if line and not line[-1]:
-			warnings.warn('trailing tab found on line %d: %s' % (self.lineno, self.line))
+			log.warning('trailing tab found on line %d: %s' % (self.lineno, self.line))
 			while line and not line[-1]:
 				del line[-1]
 		return line
@@ -1175,7 +1174,8 @@ def main(args=None, font=None):
 		del args[0]
 	for f in args:
 		log.debug("Processing %s", f)
-		table = build(open(f, 'rt', encoding="utf-8"), font, tableTag=tableTag)
+		with open(f, 'rt', encoding="utf-8") as f:
+			table = build(f, font, tableTag=tableTag)
 		blob = table.compile(font) # Make sure it compiles
 		decompiled = table.__class__()
 		decompiled.decompile(blob, font) # Make sure it decompiles!
